Option Strict On
Option Explicit On

Imports System.IO
Imports System.Text
Imports System.Diagnostics

Friend Class MainForm

#Region " Enumerations "

    'Propriedades das colunas do Grid
    Private Enum GridColumns
        Name
        DataType
        IsReadOnly
        IsWriteOnly
        IsArray
        IsList
    End Enum

#End Region

#Region " Private Variables And Constants "

    Private _propertyCounter As Int32
    Private Const StringDataType As String = "String"

#End Region

#Region " MainForm Event Handlers "

    Private Sub MainForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        'salva preferencias do usurio
        If My.Settings.Prefix.Equals("m", StringComparison.InvariantCultureIgnoreCase) Then
            mRadioButton.Checked = True
        Else
            _RadioButton.Checked = True
        End If
        OpenFileCheckBox.Checked = My.Settings.OpenFile

        'Adiciona primivitas e tipos de dados vb
        Dim dataTypeList As New List(Of String)
        With dataTypeList
            .Add("Boolean")
            .Add("Byte")
            .Add("Char")
            .Add("Date")
            .Add("DateTime")
            .Add("Decimal")
            .Add("Double")
            .Add("Int16")
            .Add("Int32")
            .Add("Int64")
            .Add("Integer")
            .Add("Long")
            .Add("Object")
            .Add("SByte")
            .Add("Short")
            .Add("Single")
            .Add(StringDataType)
            .Add("UInt16")
            .Add("UInt32")
            .Add("UInt64")
        End With

        Dim dataTypeColumn As DataGridViewComboBoxColumn = CType(PropertiesDataGridView.Columns(GridColumns.DataType), DataGridViewComboBoxColumn)
        dataTypeColumn.DataSource = dataTypeList
        dataTypeColumn.DefaultCellStyle.NullValue = StringDataType
        MemberVariableBindingSource.AddNew()

    End Sub

    Private Sub MainForm_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        'salva preferencais do usurio
        My.Settings.Prefix = IIf(mRadioButton.Checked, "m", "_").ToString
        My.Settings.OpenFile = OpenFileCheckBox.Checked
        My.Settings.Save()
    End Sub

    Private Sub AddNewButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AddNewButton.Click
        MemberVariableBindingSource.AddNew()
    End Sub

    Private Sub DeleteButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DeleteButton.Click
        If IsNothing(MemberVariableBindingSource.Current) Then Return
        MemberVariableBindingSource.Remove(MemberVariableBindingSource.Current)
    End Sub

    Private Sub GenerateClassButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles GenerateClassButton.Click

        Try

            Me.Cursor = Cursors.WaitCursor

            Dim className As String = Me.ClassNameTextBox.Text.Trim
            If className = String.Empty Then
                Me.Cursor = Cursors.Default
                MessageBox.Show("Informe o nome da classe.", Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Warning)
                Return
            End If

            Dim isAtleastOneProperty As Boolean
            Dim isAtleastOneListProperty As Boolean
            Dim memberVariable As MemberVariable = Nothing
            Dim openFile As Boolean = OpenFileCheckBox.Checked
            Dim removeMemberVariableList As New List(Of MemberVariable)
            Dim privateVariablePrefix As String = IIf(mRadioButton.Checked, "m", "_").ToString

            MemberVariableBindingSource.EndEdit()

            'Validata variaveis membros
            For Each row As DataGridViewRow In PropertiesDataGridView.Rows

                memberVariable = CType(row.DataBoundItem, MemberVariable)

                If IsNothing(memberVariable.Name) OrElse String.IsNullOrEmpty(memberVariable.Name) OrElse memberVariable.Name.Trim = String.Empty Then
                    removeMemberVariableList.Add(memberVariable)
                Else

                    memberVariable.Name = memberVariable.Name.Trim

                    If IsNothing(memberVariable.DataType) Then
                        memberVariable.DataType = StringDataType
                    End If

                    If memberVariable.IsList Then
                        isAtleastOneListProperty = True
                    End If

                    isAtleastOneProperty = True

                End If

            Next

            'Remove variaveis membros invlidos
            For Each item As MemberVariable In removeMemberVariableList
                MemberVariableBindingSource.Remove(item)
            Next

            MemberVariableBindingSource.EndEdit()

            Dim classBuilder As New StringBuilder

            'Cria a classe
            With classBuilder

                .Append(String.Format("'{0}{1}", New String(Chr(45), 60), Environment.NewLine))
                .Append(String.Format("' <auto-generated>{0}", Environment.NewLine))
                .Append(String.Format("' {0}Este Cdigo foi gerador pela ferramenta : GeradorClassesVBNET.{1}", vbTab, Environment.NewLine))
                .Append(String.Format("' {0}Cdigo adaptado por : Macoratti .net (Autor = Hemanth Kumar) {1}", vbTab, Environment.NewLine))
                .Append(String.Format("' {0}Site:http://www.macoratti.net{1}", vbTab, Environment.NewLine))
                .Append(String.Format("' </auto-generated>{0}", Environment.NewLine))
                .Append(String.Format("'{0}{1}{1}", New String(Chr(45), 60), Environment.NewLine))

                .Append(String.Format("Option Strict On{0}", Environment.NewLine))
                .Append(String.Format("Option Explicit On{0}{0}", Environment.NewLine))

                If isAtleastOneListProperty Then
                    .Append(String.Format("Imports System.Collections.Generic{0}{0}", Environment.NewLine))
                End If

                .Append(String.Format("Public Class {0}{1}", className, Environment.NewLine))

                If isAtleastOneProperty Then

                    .Append(String.Format("{1}#Region {0} Private Variables {0}{1}{1}", Chr(34), Environment.NewLine))

                    Dim privateVariable As String
                    Dim dataType As String
                    Dim propertySpecifier As String = String.Empty

                    'cria as variveis privadas
                    For Each row As DataGridViewRow In PropertiesDataGridView.Rows

                        memberVariable = CType(row.DataBoundItem, MemberVariable)
                        privateVariable = GetPrivateVariable(memberVariable.Name, privateVariablePrefix)
                        dataType = GetDataType(memberVariable)
                        .Append(String.Format("{0}Private {1} As {2}{3}", vbTab, privateVariable, dataType, Environment.NewLine))

                    Next

                    .Append(String.Format("{0}#End Region{0}", Environment.NewLine))

                    'cria a propriedade
                    For Each row As DataGridViewRow In PropertiesDataGridView.Rows

                        memberVariable = CType(row.DataBoundItem, MemberVariable)
                        privateVariable = GetPrivateVariable(memberVariable.Name, privateVariablePrefix)
                        dataType = GetDataType(memberVariable)

                        If memberVariable.IsReadOnly Then
                            propertySpecifier = "ReadOnly "
                        ElseIf memberVariable.IsWriteOnly Then
                            propertySpecifier = "WriteOnly "
                        Else
                            propertySpecifier = String.Empty
                        End If

                        .Append(String.Format("{4}{0}Public {1}Property {2}() As {3}{4}", vbTab, propertySpecifier, memberVariable.Name, dataType, Environment.NewLine))

                        If Not memberVariable.IsWriteOnly Then
                            .Append(String.Format("{0}{0}Get{1}", vbTab, Environment.NewLine))
                            .Append(String.Format("{0}{0}{0}Return {1}{2}", vbTab, privateVariable, Environment.NewLine))
                            .Append(String.Format("{0}{0}End Get{1}", vbTab, Environment.NewLine))
                        End If

                        If Not memberVariable.IsReadOnly Then
                            .Append(String.Format("{0}{0}Set(ByVal value As {1}){2}", vbTab, dataType, Environment.NewLine))
                            .Append(String.Format("{0}{0}{0}{1} = value{2}", vbTab, privateVariable, Environment.NewLine))
                            .Append(String.Format("{0}{0}End Set{1}", vbTab, Environment.NewLine))
                        End If

                        .Append(String.Format("{0}End Property{1}", vbTab, Environment.NewLine))

                    Next

                End If

                .Append(String.Format("{0}End Class", Environment.NewLine))

            End With

            'salva o arquivo de classe
            Dim classFile As String = Path.Combine(My.Computer.FileSystem.SpecialDirectories.MyDocuments, String.Format("{0}.vb", className))
            Dim writer As StreamWriter = File.CreateText(classFile)
            writer.Write(classBuilder.ToString)
            writer.Close()
            writer.Dispose()

            Me.Cursor = Cursors.Default
            MessageBox.Show(String.Format("O arquivo da classe foi gerado e salvo em  '{0}'.", classFile), Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Information)

            If openFile Then
                Process.Start("notepad.exe", classFile)
            End If

        Catch ex As Exception
            Me.Cursor = Cursors.Default
            MessageBox.Show(String.Format("Erro no trato no gerador de classes :{0}{0}{1}", Environment.NewLine, ex.Message), "Unhandled Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try

    End Sub

#End Region

#Region " MemberVariableBindingSource Event Handlers "

    Private Sub MemberVariableBindingSource_AddingNew(ByVal sender As Object, ByVal e As System.ComponentModel.AddingNewEventArgs) Handles MemberVariableBindingSource.AddingNew

        Dim memberVariable As New MemberVariable
        _propertyCounter = _propertyCounter + 1
        memberVariable.Name = String.Format("Property{0}", _propertyCounter)
        e.NewObject = memberVariable

    End Sub

#End Region

#Region " PropertiesDataGridView Event Handlers "

    Private Sub PropertiesDataGridView_CellEndEdit(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewCellEventArgs) Handles PropertiesDataGridView.CellEndEdit

        MemberVariableBindingSource.EndEdit()

        If IsNothing(PropertiesDataGridView.CurrentRow) OrElse IsNothing(PropertiesDataGridView.CurrentCell) Then
            Return
        End If

        If PropertiesDataGridView.CurrentCell.ColumnIndex = GridColumns.Name OrElse PropertiesDataGridView.CurrentCell.ColumnIndex = GridColumns.DataType Then
            Return
        End If

        If Not CType(PropertiesDataGridView.CurrentCell.Value, Boolean) Then
            Return
        End If

        Select Case PropertiesDataGridView.CurrentCell.ColumnIndex
            Case GridColumns.IsReadOnly
                PropertiesDataGridView.CurrentRow.Cells(GridColumns.IsWriteOnly).Value = False
            Case GridColumns.IsWriteOnly
                PropertiesDataGridView.CurrentRow.Cells(GridColumns.IsReadOnly).Value = False
            Case GridColumns.IsArray
                PropertiesDataGridView.CurrentRow.Cells(GridColumns.IsList).Value = False
            Case GridColumns.IsList
                PropertiesDataGridView.CurrentRow.Cells(GridColumns.IsArray).Value = False
        End Select

        MemberVariableBindingSource.EndEdit()

    End Sub

#End Region

#Region " Private Methods "

    Private Function GetPrivateVariable(ByVal value As String, ByVal prefix As String) As String

        If prefix.Equals("m", StringComparison.InvariantCultureIgnoreCase) Then
            Return String.Format("m{0}", value)
        Else
            Return String.Format("_{0}{1}", value.Chars(0).ToString.ToLower, Microsoft.VisualBasic.Right(value, value.Length - 1))
        End If

    End Function

    Private Function GetDataType(ByVal memberVariable As MemberVariable) As String

        If Not memberVariable.IsArray And Not memberVariable.IsList Then
            Return memberVariable.DataType
        ElseIf memberVariable.IsArray Then
            Return String.Format("{0}()", memberVariable.DataType)
        ElseIf memberVariable.IsList Then
            Return String.Format("List(Of {0})", memberVariable.DataType)
        End If

        Return String.Empty

    End Function

#End Region
   
End Class